function [FE,meanlt,jitter,lt]=APstats_v2(modelname,I0,nrep,Na_max,scaling)

% APstats calculates spike statistics for multiple repetitions of a
% stimulus applied to a stochastic Hodgkin-Huxley model from:
%
%    Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%    approximation of ion channel gating models," Annals of Biomedical
%    Engineering 37(4):824838.
%
% [FE,meanlt,jitter,lt]=APstats_v2(modelname,I0,nrep,Na_max,scaling)
%
% FE is the firing efficiency, i.e., the fraction of trials in which a
%    spike is generated.
% meanlt is the mean latency of spikes in units of millisecond.
% jitter is the standard deviation of spike latencies in units of millisecond.
% lt is the spike latencies in units of millisecond.
%
% modelname is the model type, which may be one of the following:-
%    - 'fx':      Fox algorithm, standard sodium channel.
%    - 'cw':      Chow & White algorithm, standard sodium channel.
%    - 'fxsm':    Fox algorithm, single m-stochastic h sodium channel.
%    - 'cwsm':    Chow & White algorithm, single m-stochastic h channel.
%    - 'fxsmdh':  Fox algorithm, single m-deterministic h sodium channel.
%    - 'cwsmdh':  Chow & White algorithm, single m-deterministic h channel.
%    - 'fxemp':   Fox algorithm, standard sodium channel with empirically
%                    -derived noise term statistics.
% I0 is the current amplitude in units of picoampere.  A single value
%    (e.g., I0 = 22) produces a single pulse of duration 100 microsecond
%    and amplitude I0 pA.
%    A pair of values (e.g., I0 = [22 35]) produces a conditioning prepulse
%    of duration 500 microsecond & amplitude I0(1) pA and a pulse of
%    duration 100 microsecond & amplitude I0(2) pA.
% nrep is the number of repetitions (trials)
% Na_max is the total number of sodium channels
% scaling determines whether or not the membrane area is scaled with Na_max
%    - 'scaled':    produces scaling of the area by Na_max/1e3 with
%                   constant channel density
%    - 'notscaled': produces to a constant area with channel density
%                   dependent on Na_max
%
% Ian C. Bruce (ibruce@ieee.org)  2007 - 2009

% Create vectors for spike results
sp = zeros(1,nrep); % Spike occurences
lt_n = zeros(1,nrep); % Latencies in terms of time bin number
lt = zeros(1,nrep); % Latencies in terms of time in milliseconds

%Setup the waitbar.
% wb1 = waitbar(0,['Running ' modelname '...']);
% set(wb1,'name','APstats_v2 - 0%');

% Repeat simulation nrep times

% h1 = figure;

for k = 1:nrep

    [V,N_Na,Istim,t] = stochasticHH_v2(modelname,I0,Na_max,scaling); % Run the chosen model
    
    sp(k) = max(V) > 45; % A spike has occurred if the peak relative
                          % membrane potential exceeds 45 mV
                          
    if sp(k)              % If a spike has occurred, determine latency
        lt_n(k) = find(V == max(V));
        lt(k) = t(lt_n(k));
    else              % Latency not defined, if no spike has occurred
        lt_n(k) = NaN;
        lt(k) = NaN;
    end
    
    
    if exist('h1')
        figure(h1)
        if sp(k) == 1
            plot(t,V,'r')
        else
            plot(t,V,'k')
        end
        hold on
    end
    
    %UPDATE WAIT-BAR every 1%
%     if (mod(k,(round(nrep/100))) == 0)
%         waitbar(k/nrep,wb1)
%         set(wb1,'name',['APstats_v2 - ' sprintf('%2.1f',k/nrep*100) '%'])
%     end
    
end

%Close Waitbar
% close(wb1)

FE = sum(sp)/nrep; % Calculate firing efficiency
if FE>0 % If any spikes have been generated, calculate mean latency and jitter
    meanlt = mean(lt(find(~isnan(lt))));
    jitter = std(lt(find(~isnan(lt))));
else % Mean latency and jitter not defined, if no spikes have occurred
    meanlt = NaN;
    jitter = NaN;
end
