function [m,gm,h,gh,n,gn,t] = evalFox(V,sim_time,Na_max,K_max)

% evalFox runs a simulation of a stochastic Hodgkin-Huxley model using
% the Chow & White (1996) algorithm and calculates the Fox & Lu (1994)
% noise term value that is required to track the Chow & White gating
% particle states at each time step.
%
% For more details, see:
%
%    Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%    approximation of ion channel gating models," Annals of Biomedical
%    Engineering 37(4):824838.
%
% [m,gm,h,gh,n,gn,t] = evalFox(V,sim_time,Na_max,K_max)
%
% m is the sodium activation particle variable
% gm is the sodium activation particle noise term
% h is the sodium inactivation particle variable
% gh is the sodium inactivation particle noise term
% n is the potassium activation particle variable
% gn is the potassium activation particle noise term
% t is the time in units of millisecond.
%
% V is the relative transmembrane potential in units of millivolt.
% sim_time is the simulation time in units of millisecond.
% Na_max is the number of sodium channels.
% K_max is the number of potassium channels.
%
% Ian C. Bruce (ibruce@ieee.org), Faheem Dinath & Melissa T. Perri  2007

dt = 1e-3; % Euler integration step in ms [dt = 1us]
t = 0:dt:sim_time; % time vector in ms
max_k = length(t); % number of Euler integration steps

% Create output variables
m = zeros(1,max_k+1); % m-particle vector
gm = zeros(1,max_k);  % m-particle noise-term vector
h = zeros(1,max_k+1); % h-particle vector
gh = zeros(1,max_k);  % h-particle noise-term vector
n = zeros(1,max_k+1); % n-particle vector
gn = zeros(1,max_k);  % n-particle noise-term vector

% Calculate gating particle transition rates
markovrates = calcrates(V);

% Calculate initial channel states
[Nmh, Nn] = initcw(markovrates,Na_max,K_max,dt);

% Calculate initial equivalent gating particle values
m(1) = ((Nmh(4) + Nmh(8))/Na_max)^(1/3);           % Eq. (13)
h(1) = (Nmh(5) + Nmh(6) + Nmh(7) + Nmh(8))/Na_max; % Eq. (14)
n(1) = (Nn(5)/K_max)^(1/4);                        % Eq. (12)

%Setup the waitbar.
% wb1 = waitbar(0,'Running...');
% set(wb1,'name','evalFox - 0%');

for k = 1:max_k

    % Calculate channel states for next Euler time step
    [Nmh, Nn] = cw(Nmh,Nn,markovrates,dt);   

    % Calculate next equivalent gating particle values
    m(k+1) = ((Nmh(4) + Nmh(8))/Na_max)^(1/3); % Eq. (13)
    h(k+1) = sum(Nmh(5:8))/Na_max;             % Eq. (14)
    n(k+1) = (Nn(5)/K_max)^(1/4);              % Eq. (12)
    
    % Calculate equivalent gating particle noise terms - Eq. (15)
    gm(k) = m(k+1) - m(k) - (markovrates.am * (1-m(k)) - markovrates.bm * m(k)) * dt;
    gh(k) = h(k+1) - h(k) - (markovrates.ah * (1-h(k)) - markovrates.bh * h(k)) * dt;
    gn(k) = n(k+1) - n(k) - (markovrates.an * (1-n(k)) - markovrates.bn * n(k)) * dt;   

    %UPDATE WAIT-BAR every 1%
%     if (mod(k,(round(max_k/100))) == 0)
%         waitbar(k/max_k,wb1)
%         set(wb1,'name',['evalFox - ' sprintf('%2.1f',k/max_k*100) '%'])
%     end
    
end

%Close Waitbar
% close(wb1)

% Truncate gating particle vectors to make them the same length as the
% noise term vectors
m = m(1:end-1);
n = n(1:end-1);
h = h(1:end-1);

% end of main function evalFox

function markovrates=calcrates(V)
% Calculate gating particle transition rates

    % Calculate sodium gating particle transition rates
    markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));   % Eq. (18)
    markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41)); % Eq. (19)
    markovrates.ah = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));  % Eq. (20)
    markovrates.bh = (22.57)/(1 + exp((56.0 - V)/12.5));                % Eq. (21)

    % Calculate potassium gating particle transition rates
    if (V == 35)
        markovrates.an = (129/100); % limit((0.129*(V - 35))/(1 - exp((35 - V)/10)),V,35)
        markovrates.bn = (809/250); % limit((0.3236*(35 - V))/(1 - exp((V - 35)/10)),V,35)
    else
        markovrates.an = (0.129*(V - 35))/(1 - exp((35 - V)/10));  % Eq. (22)
        markovrates.bn = (0.3236*(35 - V))/(1 - exp((V - 35)/10)); % Eq. (23)
    end
    
% end of subfunction calcrates

function [Nmh,Nn]=initcw(markovrates,Na_max,K_max,dt)
% Randomize initial channel states
  
    % Calculate initial steady-state fraction of open m & h particles
    minf = markovrates.am/(markovrates.am+markovrates.bm);
    hinf = markovrates.ah/(markovrates.ah+markovrates.bh);
    % Set up initial distribution of sodium channel states   
    Nmh = zeros(1,8);
    Nmh(8) = round(Na_max*minf^3*hinf);
    Nmh(5) = round(Na_max*(1-minf^3)*hinf);
    Nmh(4) = round(Na_max*minf^3*(1-hinf));
    Nmh(1) = Na_max - sum(Nmh([4 5 8]));
    
    % Calculate initial steady-state fraction of open n particles
    ninf = markovrates.an/(markovrates.an+markovrates.bn);
    % Set up initial distribution of potassium channel states   
    Nn = zeros(1,5);
    Nn(5) = round(K_max*ninf^4);
    Nn(1) = K_max - Nn(5);
    
    settletime = 500; % Number of Euluer time steps over which to randomize initial conditions

    for lp=1:settletime
        
        % Update channel states
        [Nmh, Nn] = cw(Nmh,Nn,markovrates,dt);   
        
    end

% end of subfunction initcw

function [Nmh,Nn]=cw(Nmh,Nn,markovrates,dt)
% Calculate channel states after one Euler time step using the Chow & White
% algorithm

    % Calculate transition rates of "escapes"
    zeta = [3*markovrates.am + markovrates.ah;...
            markovrates.bm + markovrates.ah + 2*markovrates.am;...
            2*markovrates.bm + markovrates.ah + markovrates.am;...
            3*markovrates.bm + markovrates.ah;...
            3*markovrates.am + markovrates.bh;...
            markovrates.bm + markovrates.bh + 2*markovrates.am;...
            2*markovrates.bm + markovrates.bh + markovrates.am;...
            3*markovrates.bm + markovrates.bh;...
            4*markovrates.an;...
            3*markovrates.an + markovrates.bn;...
            2*markovrates.an + 2*markovrates.bn;...
            markovrates.an + 3*markovrates.bn;...
            4*markovrates.bn]; % Based on Eq. (A3) of Mino et al. (ABME 2002)

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2,2,3,3,4,1,5,2,6,3,7,4,8,5,6,6,7,7,8,1,2,2,3,3,4,4,5];
    p_trans_next = [0,2,1,3,2,4,3,5,1,6,2,7,3,8,4,6,5,7,6,8,7,2,1,3,2,4,3,5,4];
        
    t_l_total = 0; % cumulative transition lifetime for this Euler time step
        
    while (t_l_total<=dt)
                
        lambda = [Nmh Nn]*zeta; % Based on Eq. (A2) of Mino et al. (ABME 2002)
        
        % Determine transition lifetime
        t_l = - log(rand(1)) / lambda; % Eq. (A4) of Mino et al. (ABME 2002)
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l;
        
        % If cumulative transition lifetime is longer than the Euler time
        % step, then return number of channels presently in each state
        if (t_l_total>dt)
            
            break;
            
        end
        
        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            3*markovrates.am*Nmh(1),...
            markovrates.bm*Nmh(2),...
            2*markovrates.am*Nmh(2),...
            2*markovrates.bm*Nmh(3),...
            markovrates.am*Nmh(3),...
            3*markovrates.bm*Nmh(4),...
            markovrates.ah*Nmh(1),...
            markovrates.bh*Nmh(5),...
            markovrates.ah*Nmh(2),...
            markovrates.bh*Nmh(6),...
            markovrates.ah*Nmh(3),...
            markovrates.bh*Nmh(7),...
            markovrates.ah*Nmh(4),...
            markovrates.bh*Nmh(8),...
            3*markovrates.am*Nmh(5),...
            markovrates.bm*Nmh(6),...
            2*markovrates.am*Nmh(6),...
            2*markovrates.bm*Nmh(7),...
            markovrates.am*Nmh(7),...
            3*markovrates.bm*Nmh(8),...
            4*markovrates.an*Nn(1),...
            markovrates.bn*Nn(2),...
            3*markovrates.an*Nn(2),...
            2*markovrates.bn*Nn(3),...
            2*markovrates.an*Nn(3),...
            3*markovrates.bn*Nn(4),...
            markovrates.an*Nn(4),...
            4*markovrates.bn*Nn(5)]/lambda); % Based on Eqs. (A5) & (A6) of Mino et al. (ABME 2002)

        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        if (ind < 22)
            Nmh(p_trans_curr(ind)) = Nmh(p_trans_curr(ind)) - 1; % decrease number of channels in current state
            Nmh(p_trans_next(ind)) = Nmh(p_trans_next(ind)) + 1; % increase number of channels in next state
        else
            Nn(p_trans_curr(ind)) = Nn(p_trans_curr(ind)) - 1; % decrease number of channels in current state
            Nn(p_trans_next(ind)) = Nn(p_trans_next(ind)) + 1; % increase number of channels in next state
        end
        
    end

% end of subfunction cw
