% fig3.m  Run simulations and plot results for Fig. 3 of:
%
%            Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%            approximation of ion channel gating models," Annals of Biomedical
%            Engineering 37(4):824838.
%
% Ian C. Bruce (ibruce@ieee.org)  2009

%% Set up parameters
V = 17;         % Relative transmembrane potential in mV
sim_time = 4e3; % Simulation duration in ms
Na_max = 1e3;   % Number of sodium channels
K_max = 333;    % Number of potassium channels

%% Run simulation
[m,gm,h,gh,n,gn,t] = evalFox(V,sim_time,Na_max,K_max);

save fig3_results

%% Analyze results and plot

dt = t(2)-t(1);

% Calculate amplitude histograms for noise terms
mmax = max(abs(gm));
hmax = max(abs(gh));
nmax = max(abs(gn));

medges = linspace(-1.1*mmax,1.1*mmax,50);
hedges = linspace(-1.1*hmax,1.1*hmax,50);
nedges = linspace(-1.1*nmax,1.1*nmax,50);

mbinsize = medges(2)-medges(1);
hbinsize = hedges(2)-hedges(1);
nbinsize = nedges(2)-nedges(1);

mamps = histc(gm,medges);
hamps = histc(gh,hedges);
namps = histc(gn,nedges);

markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));
markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41));
markovrates.ah = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));
markovrates.bh = (22.57)/(1 + exp((56.0 - V)/12.5));

if (V == 35)
    markovrates.an = (129/100); % limit((0.129*(V - 35))/(1 - exp((35 - V)/10)),V,35)
    markovrates.bn = (809/250); % limit((0.3236*(35 - V))/(1 - exp((V - 35)/10)),V,35)
else
    markovrates.an = (0.129*(V - 35))/(1 - exp((35 - V)/10));
    markovrates.bn = (0.3236*(35 - V))/(1 - exp((V - 35)/10));
end

mstd = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm))*sqrt(dt);
hstd = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh))*sqrt(dt);
nstd = sqrt((2/K_max)*(markovrates.an*markovrates.bn)/(markovrates.an+markovrates.bn))*sqrt(dt);

mpoints = linspace(-1.1*mmax,1.1*mmax,101);
hpoints = linspace(-1.1*hmax,1.1*hmax,101);
npoints = linspace(-1.1*nmax,1.1*nmax,101);

mstepsize = mpoints(2)-mpoints(1);
hstepsize = hpoints(2)-hpoints(1);
nstepsize = npoints(2)-npoints(1);

mdist = gaussian(mpoints,0,mstd);
hdist = gaussian(hpoints,0,hstd);
ndist = gaussian(npoints,0,nstd);

% Calculate autocorrelation functions for noise terms
[Rm,mlag] = xcorr(gm,40,'coeff');
[Rh,hlag] = xcorr(gh,40,'coeff');
[Rn,nlag] = xcorr(gn,40,'coeff');

mdur = 0.2; % Duration in ms of m-particle simulation results to be plotted
hdur = 20;  % Duration in ms of h-particle simulation results to be plotted
ndur = 200; % Duration in ms of n-particle simulation results to be plotted

if (sim_time < mdur)
    error('The variable sim_time (=%0.3g) is less than the variable mdur (=%0.3g)\nIncrease sim_time or decrease mdur',sim_time,mdur)
elseif (sim_time < hdur)
    error('The variable sim_time (=%0.3g) is less than the variable hdur (=%0.3g)\nIncrease sim_time or decrease hdur',sim_time,hdur)
elseif (sim_time < ndur)
    error('The variable sim_time (=%0.3g) is less than the variable ndur (=%0.3g)\nIncrease sim_time or decrease ndur',sim_time,ndur)
end

mlen = round(mdur/dt);
hlen = round(hdur/dt);
nlen = round(ndur/dt);

f1=figure;
set(f1,'PaperPosition',[0.5 1 7.5 6.5])

wysiwyg(f1)

FontName = 'Times';

% Gating particle time series plots

ax1=subplot(4,3,1);
plot(t(1:mlen),m(1:mlen),'k','linewidth',1.0)
set(ax1,'FontName',FontName)
title('Sodium activation')
ylabel('m')
xlabel('Time (ms)')

ax2=subplot(4,3,2);
plot(t(1:hlen),h(1:hlen),'k','linewidth',1.0)
set(ax2,'FontName',FontName)
title('Sodium inactivation')
ylabel('h')
xlabel('Time (ms)')

ax3=subplot(4,3,3);
plot(t(1:nlen),n(1:nlen),'k','linewidth',1.0)
set(ax3,'FontName',FontName)
title('Potassium activation')
ylabel('n')
xlabel('Time (ms)')

% Noise term time series plots

ax4=subplot(4,3,4);
plot(t(1:mlen),gm(1:mlen),'k','linewidth',1.0)
set(ax4,'FontName',FontName)
ylabel('\Deltag_m')
xlabel('Time (ms)')

ax5=subplot(4,3,5);
plot(t(1:hlen),gh(1:hlen),'k','linewidth',1.0)
set(ax5,'FontName',FontName)
ylabel('\Deltag_h')
xlabel('Time (ms)')

ax6=subplot(4,3,6);
plot(t(1:nlen),gn(1:nlen),'k','linewidth',1.0)
set(ax6,'FontName',FontName)
ylabel('\Deltag_n')
xlabel('Time (ms)')

% Amplitude histogram plots

ax7=subplot(4,3,7);
h1a=fill(mpoints,mdist/sum(mdist*mstepsize)*sum(mamps*mbinsize),'b');
hold on
h1b=bar(medges,mamps,'histc');
set(ax7,'FontName',FontName)
set(h1a,'FaceColor','b','EdgeColor','none')
set(h1b,'FaceColor','k','EdgeColor','k')
ylabel('# of occurrences')
xlabel('\Deltag_m')

ax8=subplot(4,3,8);
h2a=fill(hpoints,hdist/sum(hdist*hstepsize)*sum(hamps*hbinsize),'b');
hold on
h2b=bar(hedges,hamps,'histc');
set(ax8,'FontName',FontName)
set(h2a,'FaceColor','b','EdgeColor','none')
set(h2b,'FaceColor','k','EdgeColor','k')
ylabel('# of occurrences')
xlabel('\Deltag_h')

ax9=subplot(4,3,9);
h3a=fill(npoints,ndist/sum(ndist*nstepsize)*sum(namps*nbinsize),'b');
hold on
h3b=bar(nedges,namps,'histc');
set(ax9,'FontName',FontName)
set(h3a,'FaceColor','b','EdgeColor','none')
set(h3b,'FaceColor','k','EdgeColor','k')
ylabel('# of occurrences')
xlabel('\Deltag_n')

% Autocorrelation function plots

ax10=subplot(4,3,10);
plot(mlag,Rm,'k','linewidth',1.5)
ylim([-0.2 1.1])
set(ax10,'FontName',FontName)
ylabel('R_m')
xlabel('Lag (\mus)')

ax11=subplot(4,3,11);
plot(hlag,Rh,'k','linewidth',1.5)
ylim([-0.2 1.1])
set(ax11,'FontName',FontName)
ylabel('R_h')
xlabel('Lag (\mus)')

ax12=subplot(4,3,12);
plot(nlag,Rn,'k','linewidth',1.5)
ylim([-0.2 1.1])
set(ax12,'FontName',FontName)
ylabel('R_n')
xlabel('Lag (\mus)')