% figs4and5.m  Run simulations and plot results for Figs. 4 & 5 of:
%
%                 Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%                 approximation of ion channel gating models," Annals of Biomedical
%                 Engineering 37(4):824838.
%
% Ian C. Bruce (ibruce@ieee.org)  2009

%% Set up parameters
Vs = [-5:1:70 72:2:84 88:4:140]; % Vector of relative transmembrane potentials in mV
Ts = [1e-3 1e-2 1e-1 1 2];
sim_time = 1e3; % Simulation duration in ms
Na_max = 1e3;   % Number of sodium channels
K_max = 333;    % Number of potassium channels

%Create analysis variable vectors
vlps = length(Vs);
tlps = length(Ts);

mmean = zeros(1,vlps);
hmean = zeros(1,vlps);
nmean = zeros(1,vlps);

mstd = zeros(tlps,vlps);
hstd = zeros(tlps,vlps);
nstd = zeros(tlps,vlps);

am = zeros(1,vlps);
bm = zeros(1,vlps);
ah = zeros(1,vlps);
bh = zeros(1,vlps);
an = zeros(1,vlps);
bn = zeros(1,vlps);

mFoxstd = zeros(1,vlps);
hFoxstd = zeros(1,vlps);
nFoxstd = zeros(1,vlps);


%% Run simulations and analyze results

for vlp=1:vlps

    disp(['Run ' num2str(vlp) '/' num2str(vlps)])
    drawnow

    V = Vs(vlp);
    [m,gm,h,gh,n,gn,t] = evalFox(V,sim_time,Na_max,K_max);

    % Calculate means and stds of noise terms

    dt = t(2)-t(1);
    if any(mod(Ts,dt)~=0)
        error('All integration periods T must be integer multiples of the simulation time step dt')
    end
    
    bins = length(gm);
    
    mmean(vlp) = mean(gm);
    hmean(vlp) = mean(gh);
    nmean(vlp) = mean(gn);

    for tlp=1:tlps

        T = Ts(tlp);
        Tbins = round(T/dt);

        numsegs = floor(bins/Tbins);       
  
        if Tbins>1

            msegs = reshape(gm(1:(numsegs*Tbins)),Tbins,numsegs);
            mstd(tlp,vlp) = std(sum(msegs)/sqrt(Tbins));

            hsegs = reshape(gh(1:(numsegs*Tbins)),Tbins,numsegs);
            hstd(tlp,vlp) = std(sum(hsegs)/sqrt(Tbins));

            nsegs = reshape(gn(1:(numsegs*Tbins)),Tbins,numsegs);
            nstd(tlp,vlp) = std(sum(nsegs)/sqrt(Tbins));
            
        else
        
            mstd(tlp,vlp) = std(gm);
            hstd(tlp,vlp) = std(gh);
            nstd(tlp,vlp) = std(gn);

        end

    end

    % Calculate theoretical stds according to Fox

    am(vlp) = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));
    bm(vlp) = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41));
    ah(vlp) = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));
    bh(vlp) = (22.57)/(1 + exp((56.0 - V)/12.5));

    if (V == 35)
        an(vlp) = (129/100); % limit((0.129*(V - 35))/(1 - exp((35 - V)/10)),V,35)
        bn(vlp) = (809/250); % limit((0.3236*(35 - V))/(1 - exp((V - 35)/10)),V,35)
    else
        an(vlp) = (0.129*(V - 35))/(1 - exp((35 - V)/10));
        bn(vlp) = (0.3236*(35 - V))/(1 - exp((V - 35)/10));
    end

    mFoxstd(vlp) = sqrt((2/Na_max)*(am(vlp)*bm(vlp))/(am(vlp)+bm(vlp)))*sqrt(dt);
    hFoxstd(vlp) = sqrt((2/Na_max)*(ah(vlp)*bh(vlp))/(ah(vlp)+bh(vlp)))*sqrt(dt);
    nFoxstd(vlp) = sqrt((2/K_max)*(an(vlp)*bn(vlp))/(an(vlp)+bn(vlp)))*sqrt(dt);

    save figs4and5_results

end


%% Plot results

% Noise term statistics versus V

f1=figure;
set(f1,'PaperPosition',[2.25 1 4 8.5])
wysiwyg(f1)

FontName = 'Times';

ax1=subplot(3,1,1);
plot(Vs,mstd(1,:),'k-','linewidth',1.0)
xlim(Vs([1 end]))
hold on
grid on
plot(Vs,mmean,'k--','linewidth',1.0)
plot(Vs,mFoxstd,'b','linewidth',1.5)
set(ax1,'FontName',FontName)
title('Sodium activation')
ylabel('std[\Deltag_m]   or    E[\Deltag_m]')
xlabel('V (mV)')
legend('std[\Deltag_m]','E[\Deltag_m]','Fox std')

ax2=subplot(3,1,2);
plot(Vs(3:5:end),hstd(1,3:5:end),'k+','linewidth',1.0)
xlim(Vs([1 end]))
hold on
grid on
plot(Vs,hmean,'k--','linewidth',1.0)
plot(Vs,hFoxstd,'b','linewidth',1.5)
set(ax2,'FontName',FontName)
title('Sodium inactivation')
ylabel('std[\Deltag_h]   or    E[\Deltag_h]')
xlabel('V (mV)')
legend('std[\Deltag_h]','E[\Deltag_h]','Fox std')

ax3=subplot(3,1,3);
plot(Vs,nstd(1,:),'k-','linewidth',1.0)
xlim(Vs([1 end]))
hold on
grid on
plot(Vs,nmean,'k--','linewidth',1.0)
plot(Vs,nFoxstd,'b','linewidth',1.5)
set(ax3,'FontName',FontName)
title('Potassium activation')
ylabel('std[\Deltag_n]   or    E[\Deltag_n]')
xlabel('V (mV)')
legend('std[\Deltag_n]','E[\Deltag_n]','Fox std')

% Noise term statistics versus V and T

f2=figure;
set(f2,'PaperPosition',[2.25 1 4 8.5])
wysiwyg(f2)

FontName = 'Times';

ax1=subplot(3,1,1);
plot(Vs,mFoxstd,'k','linewidth',1.5)
xlim(Vs([1 end]))
hold on
% grid on
plot(Vs,mstd,'-','linewidth',1.0)
set(ax1,'FontName',FontName)
title('Sodium activation')
ylabel('\sigma_{g_m}')
xlabel('V (mV)')

ax2=subplot(3,1,2);
plot(Vs,hFoxstd,'k','linewidth',1.5)
xlim(Vs([1 end]))
hold on
% grid on
plot(Vs,hstd,'-','linewidth',1.0)
set(ax2,'FontName',FontName)
title('Sodium inactivation')
ylabel('\sigma_{g_h}')
xlabel('V (mV)')

ax3=subplot(3,1,3);
plot(Vs,nFoxstd,'k','linewidth',1.5)
xlim(Vs([1 end]))
hold on
% grid on
plot(Vs,nstd,'-','linewidth',1.0)
set(ax3,'FontName',FontName)
title('Potassium activation')
ylabel('\sigma_{g_n}')
xlabel('V (mV)')
