function [Ith,RS,FEs,Is]=findIthRS(modelname,Is,nrep,Na_max,scaling)

% findIthRS finds the mean threshold current (Ith) and relative spread (RS)
% of the stochastic Hodgkin-Huxley model from:
%
%    Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%    approximation of ion channel gating models," Annals of Biomedical
%    Engineering 37(4):824838.
%
% [Ith,RS,FEs,Is]=findIthRS(modelname,Is,nrep,Na_max,scaling)
%
% Ith is the mean threshold current in units of pA.
% RS  is the relative spread, i.e., standard deviation in threhsold current
%     divided by the mean threshold current.
% FEs is a vector of firing efficiency values for each current in Is.
% Is  is a vector of current amplitudes in units of pA.
%
% modelname is the model type, which may be one of the following:-
%    - 'fx':      Fox algorithm, standard sodium channel.
%    - 'cw':      Chow & White algorithm, standard sodium channel.
%    - 'fxsm':    Fox algorithm, single m-stochastic h sodium channel.
%    - 'cwsm':    Chow & White algorithm, single m-stochastic h channel.
%    - 'fxsmdh':  Fox algorithm, single m-deterministic h sodium channel.
%    - 'cwsmdh':  Chow & White algorithm, single m-deterministic h channel.
%    - 'fxemp':   Fox algorithm, standard sodium channel with empirically
%                    -derived noise term statistics.
% Is is a vector of current amplitudes in units of pA.  If this input
%    vector is empty, then a vector will be created that is appropriate for
%    the number of channels and the scaling of R & C.
% nrep is the number of repetitions (trials)
% Na_max is the total number of sodium channels
% scaling determines whether or not the membrane area is scaled with Na_max
%    - 'scaled':    produces scaling of the area by Na_max/1e3 with
%                   constant channel density
%    - 'notscaled': produces to a constant area with channel density
%                   dependent on Na_max
%
% Ian C. Bruce (ibruce@ieee.org)  2007 - 2009

% Create vector of stimulus current amplitudes if not supplied at input

if isempty(Is)

    lps = 17;

    % The appropriate range of current amplitudes depends on the number of
    % channels and whether or not scaling of R & C is applied
    if strcmp(scaling,'scaled')==1
        if Na_max == 100
            Is = linspace(12,30,lps)*Na_max/1e3; % For Na_max = 100, scaled R & C
        elseif Na_max == 500
            Is = linspace(18,26,lps)*Na_max/1e3; % For Na_max = 500, scaled R & C
        elseif Na_max == 1e3
            Is = linspace(16,26,lps); % For Na_max = 1,000
        elseif Na_max == 5e3
            Is = linspace(20,23,lps)*Na_max/1e3; % For Na_max = 5,000, scaled R & C
        elseif Na_max == 1e4
            Is = linspace(20.5,23,lps)*Na_max/1e3; % For Na_max = 10,000, scaled R & C
        else
            error(['Do not know the appropriate range of current values for Na_max = ' int2str(Na_max)])
        end
    else
        if Na_max == 100
            Is = linspace(28,40,lps); % For Na_max = 100, R & C not scaled
        elseif Na_max == 500
            Is = linspace(20,28,lps); % For Na_max = 500, R & C not scaled
        elseif Na_max == 1e3
            Is = linspace(16,26,lps); % For Na_max = 1,000
        elseif Na_max == 5e3
            Is = linspace(14,19,lps); % For Na_max = 5,000, R & C not scaled
        elseif Na_max == 1e4
            Is = linspace(11,17,lps); % For Na_max = 10,000, R & C not scaled
        else
            error(['Do not know the appropriate range of current values for Na_max = ' int2str(Na_max)])
        end
    end

else
    
    lps = length(Is);
    
end

% Create vectors for simulation results
FEs = zeros(1,lps);
meanlts = zeros(1,lps);
jitters = zeros(1,lps);

disp('')
disp(['Model version: ' modelname])

% Run loop over stimulus current amplitudes
for lp = 1:lps

    disp(['Run ' num2str(lp) '/' num2str(lps)])
    drawnow

    [FEs(lp),meanlts(lp),jitters(lp)]=APstats_v2(modelname,Is(lp),nrep,Na_max,scaling);

end

% Find integrated-Gaussian fits to FE versus I results: Eq. (16)
X = fminsearch(@(x) intgausserr(x,Is,FEs),[mean(Is) 0.03*mean(Is)]);

Ith = X(1);
RS = X(2)/X(1); % Eq. (17)

% Display threshold and RS values for integrated-Gaussian fits
disp(' ')
disp([modelname ': Ith = ' num2str(Ith,'%4.2f') ' pA; RS = ' num2str(RS,'%5.4f')])
disp(' ')
disp(' ')

