function [V,N_Na,Istim,t] = stochasticHH_v2(modelname,I0,Na_max,scaling)

% stochasticHH runs a simulation of a stochastic Hodgkin-Huxley model of:
%
%    Bruce, I. C. (2009). "Evaluation of stochastic differential equation
%    approximation of ion channel gating models," Annals of Biomedical
%    Engineering 37(4):824838.
%
% [V,N_Na,Istim,t] = stochasticHH_v2(modelname,I0,Na_max,scaling)
%
% V is the relative transmembrane potential in units of millivolt.
% N_Na is the number of open sodium channels.
% Istim is the stimulus waveform in units of microampere.
% t is the time in units of millisecond.
%
% modelname is the model type, which may be one of the following:-
%    - 'fx':      Fox algorithm, standard sodium channel.
%    - 'cw':      Chow & White algorithm, standard sodium channel.
%    - 'fxsm':    Fox algorithm, single m-stochastic h sodium channel.
%    - 'cwsm':    Chow & White algorithm, single m-stochastic h channel.
%    - 'fxsmdh':  Fox algorithm, single m-deterministic h sodium channel.
%    - 'cwsmdh':  Chow & White algorithm, single m-deterministic h channel.
%    - 'fxemp':   Fox algorithm, standard sodium channel with empirically
%                    -derived noise term statistics.
% I0 is the current amplitude in units of picoampere.  A single value
%    (e.g., I0 = 22) produces a single pulse of duration 100 microsecond
%    and amplitude I0 pA.
%    A pair of values (e.g., I0 = [22 35]) produces a conditioning prepulse
%    of duration 500 microsecond & amplitude I0(1) pA and a pulse of
%    duration 100 microsecond & amplitude I0(2) pA.
% Na_max is the total number of sodium channels
% scaling determines whether or not the membrane area is scaled with Na_max
%    - 'scaled':    produces scaling of the area by Na_max/1e3 with
%                   constant channel density
%    - 'notscaled': produces to a constant area with channel density
%                   dependent on Na_max
%
% Ian C. Bruce (ibruce@ieee.org), Faheem Dinath & Melissa T. Perri
%    2007-2009

% Set up the appropriate functions for the chosen model
if (strcmp(modelname,'fx'))
    calcN_Na = @fx;
    initN_Na = @initfx;
    calc_markovrates = @calcrates;
    global m h;
elseif (strcmp(modelname,'cw'))
    calcN_Na = @cw;
    initN_Na = @initcw;
    calc_markovrates = @calcrates;
    global Nmh;
elseif (strcmp(modelname,'fxsm'))
    calcN_Na = @fxsm;
    initN_Na = @initfxsm;
    calc_markovrates = @calcrates_sm;
    global m h;
elseif (strcmp(modelname,'cwsm'))
    calcN_Na = @cwsm;
    initN_Na = @initcwsm;
    calc_markovrates = @calcrates_sm;
    global Nmh;
elseif (strcmp(modelname,'fxsmdh'))
    calcN_Na = @fxsmdh;
    initN_Na = @initfxsmdh;
    calc_markovrates = @calcrates_sm;
    global m h;
elseif (strcmp(modelname,'cwsmdh'))
    calcN_Na = @cwsmdh;
    initN_Na = @initcwsmdh;
    calc_markovrates = @calcrates_sm;
    global Nmh;
elseif (strcmp(modelname,'fxemp'))
    calcN_Na = @fxemp;
    initN_Na = @initfxemp;
    calc_markovrates = @calcrates;
    global m h;
else
    error(['Model type "' modelname '" unknown'])
end

% Constant variables used in simulations.  Values are given on page XY
% of Bruce (ABME 2009)
dt = 1e-3; % Euler integration step in ms [dt = 1us]
sim_time = 2; % simulation time in ms
gNa = 2.569e-8; % maximum sodium conductance in mS [gamma_Na = 25.69 pS/channel]
if strcmp(scaling,'scaled')==1 % Eq. (25)
    Cm = 0.0714e-6*Na_max/1e3; % membrane capacitance in uF [Cm = 0.0714 pF]
    Rm = 1953.49e3/Na_max*1e3; % membrane resistance in kOhms [Rm = 1.95349e3 MOhms]
else
    Cm = 0.0714e-6; % membrane capacitance in uF [Cm = 0.0714nF]
    Rm = 1953.49e3; % membrane resistance in kOhms [Rm = 1.95349e3 MOhms]
end
V_rest = -78; % membrane resting potential in mV
E_Na = 66 - V_rest; % relative sodium Nernst potential in mV

t = 0:dt:sim_time; % time vector in ms
max_k = length(t); % number of Euler integration steps

% Create output variables
V = zeros(1,max_k); % Relative transmembrane potential vector
N_Na = zeros(1,max_k); % Open sodium channel count vector
Istim = zeros(1,max_k); % Stimulus vector

% Create stimulus vector for stimuli of Mino et al. (2002) and Bruce (2007)
switch length(I0)
    case 1
        Istim(1:round(0.1/dt-1)) = I0*1e-6;       % microampere     monophasic pulse
    case 2
        Istim(1:round(0.5/dt-1)) = I0(1)*1e-6;    % microampere     preconditioning pulse
        Istim(round(0.5/dt):round(0.6/dt-1)) = I0(2)*1e-6;  % uA    monophasic pulse
    otherwise
        error('Current pulse configuration not accepted')
end

% Calculate sodium gating particle transition rates for initial (resting)
% membrane potential
markovrates = calc_markovrates(V(1));

% Calculate initial number of open sodium channels
N_Na(1) = initN_Na(V(1),markovrates,Na_max,dt);

for k = 1:(max_k-1)

    % Calculate sodium gating particle transition rates for present
    % membrane potential
    markovrates = calc_markovrates(V(k));
    
    % Calculate number of open sodium channels for next Euler time step
    N_Na(k+1) = calcN_Na(V(k),markovrates,Na_max,dt);
    
    % Calculate membrane potential for next Euler time step
    V(k+1) = V(k)+(Istim(k) - V(k)/Rm - gNa*N_Na(k)*(V(k)-E_Na))*dt/Cm;
    
end

% end of main function stochasticHH_v2

function markovrates=calcrates(V)
% Calculate sodium gating particle transition rates

    markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));   % Eq. (18)
    markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41)); % Eq. (19)
    markovrates.ah = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));  % Eq. (20)
    markovrates.bh = (22.57)/(1 + exp((56.0 - V)/12.5));                % Eq. (21)   
    
% end of subfunction calcrates

function markovrates=calcrates_sm(V)
% Calculate modified sodium channel gating particle transition rates as
% described on page XY of Bruce (ABME 2009)

    markovrates.am = (1.872*(V - 25.41))/(1 - exp((25.41 - V)/6.06));
    markovrates.bm = (3.973*(21.001 - V))/(1 - exp((V - 21.001)/9.41));

    minf = markovrates.am/(markovrates.am+markovrates.bm);
    taum = 1/(markovrates.am+markovrates.bm);
    
    markovrates.am = minf.^3/taum;
    markovrates.bm = (1-minf.^3)/taum;
        
    markovrates.ah = (-0.549*(27.74 + V))/(1 - exp((V + 27.74)/9.06));
    markovrates.bh = (22.57)/(1 + exp((56.0 - V)/12.5));

% end of subfunction calcrates_sm

function N_Na=initfx(V,markovrates,Na_max,dt)
% Randomize initial sodium channel gating particle values according to Fox
% & Lu algorithm

    global m h;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);  
       
    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        N_Na=fx(V,markovrates,Na_max,dt);
        
    end

% end of subfunction initfx

function N_Na=fx(V,markovrates,Na_max,dt)
% Calculate number of open sodium channels with nint(N_Na) after one Euler
% time step according to Fox & Lu algorithm

    global m h;

    % Calculate noise term standard deviations according to Eq. (8)
    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
      
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    m = mnew;

    hnew = -1;
    while (hnew<0)||(hnew>1)
        hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    h = hnew;
    
    N_Na = round(Na_max*m^3*h); % Eq. (9) with nint(N_Na)

% end of subfunction fx

function N_Na=initcw(V,markovrates,Na_max,dt)
% Randomize initial sodium channel states according to Chow & White
% algorithm

    global Nmh;  
   
    % Set initial number of channels in each state
    if Na_max == 1e3 % Mean steady-state values known for Na_max = 1000
        Nmh = [246     6     0     0   731    17     0     0];
        settletime = 100; % Number of Euluer time steps over which to randomize initial conditions
    else % otherwise determine approximately appropriate initial values
        % Calculate initial steady-state fraction of open h particles
        hinf = markovrates.ah/(markovrates.ah+markovrates.bh);
        % Set up initial distribution of sodium channel states  
        Nmh = zeros(1,8);
        Nmh(5) = round(Na_max*hinf);
        Nmh(1) = Na_max - Nmh(5);
        settletime = 500; % Number of Euluer time steps over which to randomize initial conditions
    end

    % Randomize initial conditions
    for lp=1:settletime
        
        N_Na=cw(V,markovrates,Na_max,dt);
        
    end

% end of subfunction initcw

function N_Na=cw(V,markovrates,Na_max,dt)
% Calculate channel states after one Euler time step using the Chow & White
% algorithm

    global Nmh;
    
    % Calculate transition rates of "escapes"
    zeta = [3*markovrates.am + markovrates.ah;...
            markovrates.bm + markovrates.ah + 2*markovrates.am;...
            2*markovrates.bm + markovrates.ah + markovrates.am;...
            3*markovrates.bm + markovrates.ah;...
            3*markovrates.am + markovrates.bh;...
            markovrates.bm + markovrates.bh + 2*markovrates.am;...
            2*markovrates.bm + markovrates.bh + markovrates.am;...
            3*markovrates.bm + markovrates.bh]; % Eq. (A3) of Mino et al. (ABME 2002)

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2,2,3,3,4,1,5,2,6,3,7,4,8,5,6,6,7,7,8];
    p_trans_next = [0,2,1,3,2,4,3,5,1,6,2,7,3,8,4,6,5,7,6,8,7];
        
    t_l_total = 0; % cumulative transition lifetime for this Euler time step
        
    while (t_l_total<=dt)
                
        lambda = Nmh*zeta; % Eq. (A2) of Mino et al. (ABME 2002)
    
        % Determine transition lifetime
        t_l = - log(rand(1)) / lambda; % Eq. (A4) of Mino et al. (ABME 2002)
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l;
        
        % If cumulative transition lifetime is longer than the Euler time
        % step, then return number of channels presently in each state
        if (t_l_total>dt)
            
            break;
            
        end
        
        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            3*markovrates.am*Nmh(1),...
            markovrates.bm*Nmh(2),...
            2*markovrates.am*Nmh(2),...
            2*markovrates.bm*Nmh(3),...
            markovrates.am*Nmh(3),...
            3*markovrates.bm*Nmh(4),...
            markovrates.ah*Nmh(1),...
            markovrates.bh*Nmh(5),...
            markovrates.ah*Nmh(2),...
            markovrates.bh*Nmh(6),...
            markovrates.ah*Nmh(3),...
            markovrates.bh*Nmh(7),...
            markovrates.ah*Nmh(4),...
            markovrates.bh*Nmh(8),...
            3*markovrates.am*Nmh(5),...
            markovrates.bm*Nmh(6),...
            2*markovrates.am*Nmh(6),...
            2*markovrates.bm*Nmh(7),...
            markovrates.am*Nmh(7),...
            3*markovrates.bm*Nmh(8)]/lambda); % Eqs. (A5) & (A6) of Mino et al. (ABME 2002)

        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        Nmh(p_trans_curr(ind)) = Nmh(p_trans_curr(ind)) - 1; % decrease number of channels in current state
        Nmh(p_trans_next(ind)) = Nmh(p_trans_next(ind)) + 1; % increase number of channels in next state
        
    end
       
    N_Na = Nmh(end); % Eq. (3)

% end of subfunction cw

function N_Na=initfxsm(V,markovrates,Na_max,dt)
% Randomize initial gating particle values according to Fox & Lu algorithm
% for modified sodium channel with a single stochastic m particle and a
% stochastic h particle

    global m h;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);
    
    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        N_Na=fxsm(V,markovrates,Na_max,dt);
        
    end

% end of subfunction initfxsm

function N_Na=fxsm(V,markovrates,Na_max,dt)
% Calculate number of open sodium channels after one Euler time step
% according to Fox & Lu algorithm for modified sodium channel with a single
% stochastic m particle and a stochastic h particle

    global m h;

    % Calculate noise term standard deviations according to Eq. (8)   
    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    m = mnew;

    hnew = -1;
    while (hnew<0)||(hnew>1)
        hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    h = hnew;
    
    N_Na = round(Na_max*m*h); % Based on Eq. (9)

% end of subfunction fxsm

function N_Na=initcwsm(V,markovrates,Na_max,dt)
% Randomize initial channel states according to Chow & White algorithm
% for modified sodium channel with a single stochastic m particle and a
% stochastic h particle

    global Nmh;  
   
    % Calculate initial steady-state fraction of open m & h particles
    minf = markovrates.am/(markovrates.am+markovrates.bm);
    hinf = markovrates.ah/(markovrates.ah+markovrates.bh);
    % Set up initial distribution of sodium channel states  
    Nmh = zeros(1,4);
    Nmh(4) = round(Na_max*minf*hinf);
    Nmh(3) = round(Na_max*(1-minf)*hinf);
    Nmh(2) = round(Na_max*minf*(1-hinf));
    Nmh(1) = Na_max - sum(Nmh(2:4));
       
    settletime = 500; % Number of Euluer time steps over which to randomize initial conditions

    % Randomize initial conditions
    for lp=1:settletime
        
        N_Na=cwsm(V,markovrates,Na_max,dt);
        
    end

% end of subfunction initcwsm

function N_Na=cwsm(V,markovrates,Na_max,dt)
% Calculate channel states after one Euler time step according to Chow &
% White algorithm for modified sodium channel with a single stochastic
% m particle and a stochastic h particle

    global Nmh

    % Calculate transition rates of "escapes"
    zeta = [markovrates.am + markovrates.ah;...
            markovrates.bm + markovrates.ah;...
            markovrates.am + markovrates.bh;...
            markovrates.bm + markovrates.bh]; % Based on Eq. (A3) of Mino et al. (ABME 2002)

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2,1,3,2,4,3,4];
    p_trans_next = [0,2,1,3,1,4,2,4,3];
        
    t_l_total = 0; % cumulative transition lifetime for this Euler time step
        
    while (t_l_total<=dt)
                
        lambda = Nmh*zeta; % Eq. (A2) of Mino et al. (ABME 2002)
    
        % Determine transition lifetime
        t_l = - log(rand(1)) / lambda; % Eq. (A4) of Mino et al. (ABME 2002)
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l;
        
        % If cumulative transition lifetime is longer than the Euler time
        % step, then return number of channels presently in each state
        if (t_l_total>dt)
            
            break;
            
        end
        
        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            markovrates.am*Nmh(1),...
            markovrates.bm*Nmh(2),...
            markovrates.ah*Nmh(1),...
            markovrates.bh*Nmh(3),...
            markovrates.ah*Nmh(2),...
            markovrates.bh*Nmh(4),...
            markovrates.am*Nmh(3),...
            markovrates.bm*Nmh(4)]/lambda); % Based on Eqs. (A5) & (A6) of Mino et al. (ABME 2002)

        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        Nmh(p_trans_curr(ind)) = Nmh(p_trans_curr(ind)) - 1; % decrease number of channels in current state
        Nmh(p_trans_next(ind)) = Nmh(p_trans_next(ind)) + 1; % increase number of channels in next state
        
    end

    N_Na = Nmh(end); % Based on Eq. (3)

% end of subfunction cwsm

function N_Na=initfxsmdh(V,markovrates,Na_max,dt)
% Randomize initial gating particle values according to Fox & Lu algorithm
% for modified sodium channel with a single stochastic m particle and a
% deterministic h particle

    global m h;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);
    
    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        N_Na=fxsmdh(V,markovrates,Na_max,dt);
        
    end
    
% end of subfunction initfxsmdh

function N_Na=fxsmdh(V,markovrates,Na_max,dt)
% Calculate number of open sodium channels after one Euler time step
% according to Fox & Lu algorithm for modified sodium channel with a single
% stochastic m particle and a deterministic h particle

    global m h;

    % Calculate noise term standard deviation according to Eq. (8)    
    m_std = sqrt((2/Na_max)*(markovrates.am*markovrates.bm)/(markovrates.am+markovrates.bm));
    
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    m = mnew;

    h = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt; % Based on Eq. (11)
    
    N_Na = round(Na_max*m*h); % Based on Eq. (9)

% end of subfunction fxsmdh

function N_Na=initcwsmdh(V,markovrates,Na_max,dt)
% Randomize initial channel states according to Chow & White algorithm
% for modified sodium channel with a single stochastic m particle and a
% deterministic h particle

    global Nm h;  
    
    % Calculate initial steady-state fraction of open m & h particles
    minf = markovrates.am/(markovrates.am+markovrates.bm);
    hinf = markovrates.ah/(markovrates.ah+markovrates.bh);
    % Set up initial distribution of sodium channel states  
    Nm = zeros(1,2);
    Nm(2) = round(Na_max*minf);
    Nm(1) = Na_max - Nm(2);
    h = hinf;
    
    settletime = 500; % Number of Euluer time steps over which to randomize initial conditions

    % Randomize initial conditions
    for lp=1:settletime
        
        N_Na=cwsmdh(V,markovrates,Na_max,dt);
        
    end

% end of subfunction initcwsmdh

function N_Na=cwsmdh(V,markovrates,Na_max,dt)
% Calculate channel states after one Euler time step according to Chow &
% White algorithm for modified sodium channel with a single stochastic
% m particle and a deterministic h particle

    global Nm h;

    % Calculate transition rates of "escapes"
    zeta = [markovrates.am; markovrates.bm]; % Based on Eq. (A3) of Mino et al. (ABME 2002)

    % Vectors describing state transitions from current to next state
    p_trans_curr = [0,1,2];
    p_trans_next = [0,2,1];
        
    t_l_total = 0; % cumulative transition lifetime for this Euler time step
        
    while (t_l_total<=dt)
                
        lambda = Nm*zeta; % Eq. (A2) of Mino et al. (ABME 2002)
    
        % Determine transition lifetime
        t_l = - log(rand(1)) / lambda; % Eq. (A2) of Mino et al. (ABME 2002)
        
        % Calculate cumulative transition lifetime
        t_l_total = t_l_total + t_l;
        
        % If cumulative transition lifetime is longer than the Euler time
        % step, then return number of channels presently in each state
        if (t_l_total>dt)
            
            break;
            
        end
        
        % Calculate cumulative probability distribution for state
        % transitions
        P = cumsum([0,...
            markovrates.am*Nm(1),...
            markovrates.bm*Nm(2)]/lambda); % Based on Eqs. (A5) & (A6) of Mino et al. (ABME 2002)

        % Determine which state transition has occurred
        ind = find(rand(1)<P,1);
        
        Nm(p_trans_curr(ind)) = Nm(p_trans_curr(ind)) - 1; % decrease number of channels in current state
        Nm(p_trans_next(ind)) = Nm(p_trans_next(ind)) + 1; % increase number of channels in next state
        
    end

    h = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt; % Based on Eq. (11)
    
    N_Na = round(Nm(end)*h); % Based on Eq. (3)

% end of subfunction cwsmdh

function N_Na=initfxemp(V,markovrates,Na_max,dt)
% Randomize initial sodium channel gating particle values according to Fox
% & Lu algorithm for model with modified noise term means and standard
% deviations obtained empirically - see pp. XY-ZY of Bruce (ABME 2009)

    global m h;
    
    % Set initial values according to deterministic steady-state
    m = markovrates.am/(markovrates.am+markovrates.bm);
    h = markovrates.ah/(markovrates.ah+markovrates.bh);  
       
    lps = 100;
       
    % Randomize initial conditions
    for lp=1:lps

        N_Na=fxemp(V,markovrates,Na_max,dt);
        
    end

% end of subfunction initfxemp

function N_Na=fxemp(V,markovrates,Na_max,dt)
% Calculate number of open sodium channels after one Euler time step
% according to Fox & Lu algorithm for model with modified noise term means
% and standard deviations obtained empirically - see pp. XY-ZY of Bruce
% (ABME 2009)

    global m h;

    % Calculate noise term standard deviation according to Eq. (8)    
    h_std = sqrt((2/Na_max)*(markovrates.ah*markovrates.bh)/(markovrates.ah+markovrates.bh));
    
    % Coefficients for fit to equivalent m-particle noise term std vs. V curve; Integrated over 64 us
    a1m =      0.1256;
    b1m =       16.56;
    c1m =       2.626;
    a2m =      0.2235;
    b2m =       15.09;
    c2m =       6.433;
    a3m =      0.1016;
    b3m =       22.85;
    c3m =       16.08;
    a4m =     0.05285;
    b4m =       44.51;
    c4m =       31.01;

    % Coefficients for fit to equivalent m-particle noise term mean vs. V curve
    p1m =   -0.008043;
    p2m =     -0.1625;
    p3m =      -10.24;
    p4m =        1781;
    p5m = -3.425e+004;
    q1m =       -44.1;
    q2m =         845;
    q3m =       -8966;
    q4m =  4.769e+004;

    % Multiple-Gaussian fit to equivalent m-particle noise term std vs. V
    % curve; Integrated over 64 us
    m_std = a1m*exp(-((V-b1m)/c1m)^2) + a2m*exp(-((V-b2m)/c2m)^2) + a3m*exp(-((V-b3m)/c3m)^2) + a4m*exp(-((V-b4m)/c4m)^2);
    % Rational function fit to equivalent m-particle noise term mean vs. V curve
    m_mn = (p1m*V^4 + p2m*V^3 + p3m*V^2 + p4m*V + p5m) / (V^4 + q1m*V^3 + q2m*V^2 + q3m*V + q4m);
    
    mnew = -1;
    while (mnew<0)||(mnew>1)
        mnew = m + ((markovrates.am*(1-m))-(markovrates.bm*m))*dt + m_std*randn(1)*sqrt(dt)+m_mn*dt; % Based on Eq. (11)
    end
    m = mnew;

    hnew = -1;
    while (hnew<0)||(hnew>1)
        hnew = h + ((markovrates.ah*(1-h))-(markovrates.bh*h))*dt + h_std*randn(1)*sqrt(dt); % Eq. (11)
    end
    h = hnew;
    
    N_Na = round(Na_max*m^3*h); % Eq. (9)

% end of subfunction fxemp

